"""URL configuration for djoser reset password functionality"""

from django.urls import path

from mitol.authentication.views.djoser_views import CustomDjoserAPIView

urlpatterns = [
    path(
        "password_reset/",
        CustomDjoserAPIView.as_view({"post": "reset_password"}),
        name="password-reset-api",
    ),
    path(
        "password_reset/confirm/",
        CustomDjoserAPIView.as_view({"post": "reset_password_confirm"}),
        name="password-reset-confirm-api",
    ),
    path(
        "set_password/",
        CustomDjoserAPIView.as_view({"post": "set_password"}),
        name="set-password-api",
    ),
]



"""
Custom serializers for overriding default djoser behaviours
"""

from django.contrib.auth import get_user_model
from djoser.conf import settings
from djoser.serializers import SendEmailResetSerializer

User = get_user_model()


class CustomSendEmailResetSerializer(SendEmailResetSerializer):
    def get_user(self, is_active=True):
        # NOTE: This directly copies the implementation of djoser.serializers.UserFunctionsMixin.get_user
        # and only changes the User query. If this method is changed in an updated Djoser
        # release, this method may need to be updated as well.
        try:
            user = User._default_manager.get(
                is_active=is_active,
                **{f"{self.email_field}__iexact": self.data.get(self.email_field, "")},
            )
            if user.has_usable_password():
                return user
        except User.DoesNotExist:
            pass
        if (
            settings.PASSWORD_RESET_SHOW_EMAIL_NOT_FOUND
            or settings.USERNAME_RESET_SHOW_EMAIL_NOT_FOUND
        ):
            self.fail("email_not_found")



"""Touchstone configuration settings"""
from urllib.parse import urlparse

from mitol.common.envs import get_bool, get_site_name, get_string
from mitol.common.settings.base import SITE_BASE_URL
from mitol.mail.settings.email import EMAIL_SUPPORT

_SITE_NAME = get_site_name()

# SAML settings
SOCIAL_AUTH_SAML_SP_ENTITY_ID = get_string(
    name="SOCIAL_AUTH_SAML_SP_ENTITY_ID", default=SITE_BASE_URL, description=""
)
SOCIAL_AUTH_SAML_SP_PUBLIC_CERT = get_string(
    name="SOCIAL_AUTH_SAML_SP_PUBLIC_CERT",
    default=None,
    description="The SAML public certificate",
)
SOCIAL_AUTH_SAML_SP_PRIVATE_KEY = get_string(
    name="SOCIAL_AUTH_SAML_SP_PRIVATE_KEY",
    default=None,
    description="The SAML private key",
)
SOCIAL_AUTH_SAML_ORG_DISPLAYNAME = get_string(
    name="SOCIAL_AUTH_SAML_ORG_DISPLAYNAME",
    default=_SITE_NAME,
    description="The SAML Organization display name",
)
SOCIAL_AUTH_SAML_CONTACT_NAME = get_string(
    name="SOCIAL_AUTH_SAML_CONTACT_NAME",
    default=f"{_SITE_NAME} Support",
    description="The SAML contact name for our app",
)
SOCIAL_AUTH_SAML_IDP_ENTITY_ID = get_string(
    name="SOCIAL_AUTH_SAML_IDP_ENTITY_ID",
    default=None,
    description="The SAML IDP entity ID",
)
SOCIAL_AUTH_SAML_IDP_URL = get_string(
    name="SOCIAL_AUTH_SAML_IDP_URL", default=None, description="The SAML IDP URL"
)
SOCIAL_AUTH_SAML_IDP_X509 = get_string(
    name="SOCIAL_AUTH_SAML_IDP_X509",
    default=False,
    description="The SAML IDP x509 certificate",
)
SOCIAL_AUTH_SAML_IDP_ATTRIBUTE_PERM_ID = get_string(
    name="SOCIAL_AUTH_SAML_IDP_ATTRIBUTE_PERM_ID",
    default=None,
    description="The IDP attribute for the user's immutable ID",
)
SOCIAL_AUTH_SAML_IDP_ATTRIBUTE_NAME = get_string(
    name="SOCIAL_AUTH_SAML_IDP_ATTRIBUTE_NAME",
    default=None,
    description="The IDP attribute for the user's name",
)
SOCIAL_AUTH_SAML_IDP_ATTRIBUTE_EMAIL = get_string(
    name="SOCIAL_AUTH_SAML_IDP_ATTRIBUTE_EMAIL",
    default=None,
    description="The IDP attribute for the user's email",
)
SOCIAL_AUTH_SAML_SECURITY_ENCRYPTED = get_bool(
    name="SOCIAL_AUTH_SAML_SECURITY_ENCRYPTED",
    default=False,
    description="If True, SMAL assertions should be encrypted",
)

SOCIAL_AUTH_SAML_ORG_INFO = {
    "en-US": {
        "name": urlparse(SITE_BASE_URL).netloc,
        "displayname": SOCIAL_AUTH_SAML_ORG_DISPLAYNAME,
        "url": SITE_BASE_URL,
    }
}
SOCIAL_AUTH_SAML_TECHNICAL_CONTACT = {
    "givenName": SOCIAL_AUTH_SAML_CONTACT_NAME,
    "emailAddress": EMAIL_SUPPORT,
}
SOCIAL_AUTH_SAML_SUPPORT_CONTACT = SOCIAL_AUTH_SAML_TECHNICAL_CONTACT
SOCIAL_AUTH_DEFAULT_IDP_KEY = "default"
SOCIAL_AUTH_SAML_ENABLED_IDPS = {
    SOCIAL_AUTH_DEFAULT_IDP_KEY: {
        "entity_id": SOCIAL_AUTH_SAML_IDP_ENTITY_ID,
        "url": SOCIAL_AUTH_SAML_IDP_URL,
        "attr_user_permanent_id": SOCIAL_AUTH_SAML_IDP_ATTRIBUTE_PERM_ID,
        "attr_username": SOCIAL_AUTH_SAML_IDP_ATTRIBUTE_PERM_ID,
        "attr_email": SOCIAL_AUTH_SAML_IDP_ATTRIBUTE_EMAIL,
        "attr_full_name": SOCIAL_AUTH_SAML_IDP_ATTRIBUTE_NAME,
        "x509cert": SOCIAL_AUTH_SAML_IDP_X509,
    }
}


SOCIAL_AUTH_SAML_SECURITY_CONFIG = {
    "wantAssertionsEncrypted": SOCIAL_AUTH_SAML_SECURITY_ENCRYPTED,
    "requestedAuthnContext": False,
}



"""
Custom views to override default djoser behaviours
"""
from anymail.message import AnymailMessage
from django.conf import settings
from django.contrib.auth import update_session_auth_hash
from django.core import mail as django_mail
from djoser.email import PasswordResetEmail as DjoserPasswordResetEmail
from djoser.utils import ActionViewMixin
from djoser.views import UserViewSet
from rest_framework import status
from rest_framework.decorators import action

from mitol.mail.api import render_email_templates, send_message


class CustomDjoserAPIView(UserViewSet, ActionViewMixin):
    """
    Overrides the post method of a Djoser view to update session after successful password change
    """

    @action(["post"], detail=False)
    def set_password(self, request, *args, **kwargs):
        """
        Overrides CustomDjoserAPIView.set_password to update the session after a successful
        password change. Without this explicit refresh, the user's session would be
        invalid and they would be logged out.
        """
        response = super().set_password(request, *args, **kwargs)
        if response.status_code in (status.HTTP_200_OK, status.HTTP_204_NO_CONTENT):
            update_session_auth_hash(self.request, self.request.user)
        return response


class CustomPasswordResetEmail(DjoserPasswordResetEmail):
    """Custom class to modify base functionality in Djoser's PasswordResetEmail class"""

    def send(self, to, *args, **kwargs):
        """
        Overrides djoser.email.PasswordResetEmail#send to use our mail API.
        """
        context = self.get_context_data()
        context.update(self.context)
        with django_mail.get_connection(
            settings.NOTIFICATION_EMAIL_BACKEND
        ) as connection:
            subject, text_body, html_body = render_email_templates(
                "password_reset", context
            )
            msg = AnymailMessage(
                subject=subject,
                body=text_body,
                to=to,
                from_email=settings.MITOL_AUTHENTICATION_FROM_EMAIL,
                connection=connection,
                headers={"Reply-To": settings.MITOL_AUTHENTICATION_REPLY_TO_EMAIL},
            )
            msg.attach_alternative(html_body, "text/html")
            send_message(msg)

    def get_context_data(self):
        """Adds base_url to the template context"""
        context = super().get_context_data()
        context["base_url"] = settings.SITE_BASE_URL
        return context



"""SAML-specific auth views"""
from django.http import HttpRequest, HttpResponse
from django.urls import reverse
from social_django.utils import load_backend, load_strategy


def saml_metadata(request: HttpRequest):
    """Display SAML configuration metadata as XML"""
    complete_url = reverse("social:complete", args=("saml",))
    saml_backend = load_backend(
        load_strategy(request), "saml", redirect_uri=complete_url
    )
    metadata, _ = saml_backend.generate_metadata_xml()
    return HttpResponse(content=metadata, content_type="text/xml")



from mitol.common.envs import get_string

# Relative URL to be used by Djoser for the link in the password reset email
# (see: http://djoser.readthedocs.io/en/stable/settings.html#password-reset-confirm-url)
PASSWORD_RESET_CONFIRM_URL = "password_reset/confirm/{uid}/{token}/"

# Djoser library settings (see: http://djoser.readthedocs.io/en/stable/settings.html)
MITOL_AUTHENTICATION_FROM_EMAIL = get_string(
    default="webmaster@localhost",
    name="MITOL_AUTHENTICATION_FROM_EMAIL",
    description="E-mail to use for the from field",
)

MITOL_AUTHENTICATION_REPLY_TO_ADDRESS = get_string(
    name="MITOL_AUTHENTICATION_REPLY_TO_ADDRESS",
    default="webmaster@localhost",
    description="E-mail to use for reply-to address of emails",
)

DJOSER = {
    "PASSWORD_RESET_CONFIRM_URL": PASSWORD_RESET_CONFIRM_URL,
    "SET_PASSWORD_RETYPE": False,
    "LOGOUT_ON_PASSWORD_CHANGE": False,
    "PASSWORD_RESET_CONFIRM_RETYPE": True,
    "PASSWORD_RESET_SHOW_EMAIL_NOT_FOUND": False,
    "EMAIL": {
        "password_reset": "mitol.authentication.views.djoser_views.CustomPasswordResetEmail"
    },
    "SERIALIZERS": {
        "password_reset": "mitol.authentication.serializers.djoser_serializers.CustomSendEmailResetSerializer"
    },
}



"""Authentication Apps"""
import os

from django.apps import AppConfig


class AuthenticationApp(AppConfig):
    """Authentication AppConfig"""

    name = "mitol.authentication"
    label = "authentication"
    verbose_name = "authentication"

    # necessary because this is a namespaced app
    path = os.path.dirname(os.path.abspath(__file__))


class TransitionalAuthenticationApp(AppConfig):
    """Authentication AppConfig for transitioning a project with an existing 'authentication' app"""

    name = "mitol.authentication"
    label = "transitional_authentication"
    verbose_name = "authentication"

    # necessary because this is a namespaced app
    path = os.path.dirname(os.path.abspath(__file__))




# DO NOT EDIT THIS FILE -- AUTOGENERATED BY PANTS
# Target: src/mitol/authentication:mitol-django-authentication

from setuptools import setup

setup(**{
    'authors': [
        'MIT Office of Open Learning <mitx-devops@mit.edu>',
    ],
    'classifiers': [
        'Natural Language :: English',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
    ],
    'description': 'MIT Open Learning django app extensions for oauth toolkit',
    'install_requires': (
        'django-anymail>=6.0',
        'django<4.0,>=2.2.12',
        'djangorestframework>=3.0.0',
        'djoser==2.1.0',
        'mitol-django-common~=2.6.0',
        'mitol-django-mail~=3.2.1',
        'social-auth-app-django>=3.1.0',
    ),
    'license': 'BSD 3-Clause License',
    'name': 'mitol-django-authentication',
    'namespace_packages': (
        'mitol',
    ),
    'package_data': {
        'mitol.authentication': (
            'CHANGELOG.md',
            'README.md',
            'py.typed',
        ),
    },
    'packages': (
        'mitol',
        'mitol.authentication',
        'mitol.authentication.migrations',
        'mitol.authentication.serializers',
        'mitol.authentication.settings',
        'mitol.authentication.urls',
        'mitol.authentication.views',
    ),
    'version': '1.5.1',
    'zip_safe': True,
})



"""URL configurations for authentication"""

from django.urls import path

from mitol.authentication.views.saml import saml_metadata

urlpatterns = [
    path("saml/metadata/", saml_metadata, name="saml-metadata"),
]
